<?php

namespace Drupal\Tests\riddler\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests Riddler main test case sensitivity.
 *
 * @group captcha
 */
class RiddlerFunctionalTest extends BrowserTestBase {

  /**
   * Drupal path of the Riddler admin page.
   */
  const RIDDLER_ADMIN_PATH = 'admin/config/people/captcha/riddler';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'test_page_test',
    'block',
    'comment',
    'captcha',
    'riddler',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A test user with administrative privileges.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp():void {
    parent::setUp();
    $this->config('system.site')->set('page.front', '/test-page')->save();
    $this->adminUser = $this->drupalCreateUser([]);
    $this->adminUser->addRole($this->createAdminRole('admin', 'admin'));
    $this->adminUser->save();
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests if installing the module, won't break the site.
   */
  public function testInstallation() {
    $session = $this->assertSession();
    $this->drupalGet('<front>');
    // Ensure the status code is success:
    $session->statusCodeEquals(200);
    // Ensure the correct test page is loaded as front page:
    $session->pageTextContains('Test page text.');
  }

  /**
   * Tests if uninstalling the module, won't break the site.
   */
  public function testUninstallation() {
    // Go to uninstallation page an uninstall riddler:
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    $this->drupalGet('/admin/modules/uninstall');
    $session->statusCodeEquals(200);
    $page->checkField('edit-uninstall-riddler');
    $page->pressButton('edit-submit');
    $session->statusCodeEquals(200);
    // Confirm uninstall:
    $page->pressButton('edit-submit');
    $session->statusCodeEquals(200);
    $session->pageTextContains('The selected modules have been uninstalled.');
    // Retest the frontpage:
    $this->drupalGet('<front>');
    // Ensure the status code is success:
    $session->statusCodeEquals(200);
    // Ensure the correct test page is loaded as front page:
    $session->pageTextContains('Test page text.');
  }

  /**
   * Method for testing the Riddler riddles administration.
   */
  public function testRiddlesAdministration() {
    $session = $this->assertSession();
    $form_values_1_question = [
      'riddler[0][question]' => 'What is your favorite primary color?',
      'riddler[0][response]' => 'red, blue, yellow',
    ];
    $form_values_2_question = [
      'riddler[0][question]' => 'What is your favorite primary color?',
      'riddler[0][response]' => 'red, blue, yellow',
      'riddler[1][question]' => 'What color is Druplicon?',
      'riddler[1][response]' => 'blue',
    ];

    $addAnotherButton = 'Add another riddle';

    // Check that the admin form exists.
    $this->drupalGet(self::RIDDLER_ADMIN_PATH);
    $session->pageTextContains('Add questions that you require users to answer.');
    $session->pageTextContains('Riddle 1 question');

    // This creates a new set of question and response.
    $this->submitForm($form_values_1_question, $addAnotherButton);
    $session->pageTextContains('Add questions that you require users to answer.');
    $session->pageTextContains('Riddle 2 question');

    $this->submitForm($form_values_2_question, 'Save configuration');
    $session->fieldExists('edit-riddler-1-question');
    $session->fieldExists('edit-riddler-1-response');
    $session->elementAttributeContains('css', '#edit-riddler-1-question', 'value', 'What color is Druplicon?');
    $session->elementAttributeContains('css', '#edit-riddler-1-response', 'value', 'blue');

    // This deletes a set of question and response.
    $this->drupalGet(self::RIDDLER_ADMIN_PATH);
    $this->submitForm($form_values_2_question, 'Remove');
    $session->fieldNotExists('edit-riddler-1-question');
    $session->fieldNotExists('edit-riddler-1-response');

    // Save the config form without the deleted riddle.
    $this->submitForm($form_values_1_question, 'Save configuration');
    $session->pageTextNotContains('Riddle 2 question');
    $session->fieldNotExists('edit-riddler-1-question');
    $session->fieldNotExists('edit-riddler-1-response');
  }

  /**
   * Testing the protection of the user log in form using riddler.
   */
  public function testCaptchaOnLoginForm() {
    $session = $this->assertSession();
    // Create user and test log in without CAPTCHA.
    $this->drupalLogout();
    $user = $this->drupalCreateUser();
    $this->drupalLogin($user);
    // Log out again.
    $this->drupalLogout();

    // Set a CAPTCHA on login form.
    /** @var \Drupal\captcha\Entity\CaptchaPoint $captcha_point */
    $captcha_point = \Drupal::entityTypeManager()
      ->getStorage('captcha_point')
      ->load('user_login_form');
    $captcha_point->setCaptchaType('riddler/Riddler');
    $captcha_point->enable()->save();

    // Check if there is a CAPTCHA on the login form (look for the title).
    $this->drupalGet('user');
    $session->pageTextContains('This question is for testing whether or not you are a human visitor and to prevent automated spam submissions.');
    $session->pageTextContains('Do you really hate Spam?');

    // Try to log in, which should fail:
    $edit = [
      'name' => $user->getDisplayName(),
      'pass' => $user->pass_raw,
      'captcha_response' => '?',
    ];
    $this->submitForm($edit, 'Log in', 'user-login-form');
    // Check for error message.
    $session->pageTextContains('The answer you entered for the CAPTCHA was not correct.');
    // Check that the user really hasn't logged in:
    $this->drupalGet('user');
    $session->fieldExists('name');
    $session->fieldExists('pass');

    // Try to log in with correct answer:
    $edit = [
      'name' => $user->getDisplayName(),
      'pass' => $user->pass_raw,
      'captcha_response' => 'Yes!',
    ];
    $this->submitForm($edit, 'Log in', 'user-login-form');

    // Check that the user is logged in now:
    $this->drupalGet('user');
    $session->fieldNotExists('name');
    $session->fieldNotExists('pass');
    $session->pageTextContains($user->getDisplayName());
  }

}
